##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Validation Exceptions

This has been moved to zope.formlib.interfaces.
"""


# this moved to zope.formlib.interfaces
from zope.formlib.interfaces import (IWidgetInputError,
                                     WidgetInputError,
                                     MissingInputError,
                                     ConversionError,
                                     InputErrors,
                                     ErrorContainer,
                                     WidgetsError,
                                     IWidget,
                                     IInputWidget,
                                     IDisplayWidget,
                                     IWidgetFactory)
